package core

import (
	"git.akyoto.dev/cli/q/src/build/ast"
	"git.akyoto.dev/cli/q/src/build/expression"
	"git.akyoto.dev/cli/q/src/build/token"
)

// Evaluate evaluates an expression.
func (f *Function) Evaluate(value *expression.Expression) error {
	left := value.Children[0]
	right := value.Children[1]

	if left.IsLeaf() && left.Token.Kind == token.Identifier {
		variable := f.variables[left.Token.Text()]
		register := variable.Register
		defer f.useVariable(variable)
		return f.Execute(value.Token, register, right)
	}

	if ast.IsFunctionCall(left) && right.IsLeaf() {
		err := f.CompileCall(left)

		if err != nil {
			return err
		}

		return f.Execute(value.Token, f.cpu.Output[0], right)
	}

	tmp := f.cpu.MustFindFree(f.cpu.General)
	err := f.ExpressionToRegister(left, tmp)

	if err != nil {
		return err
	}

	f.cpu.Use(tmp)
	defer f.cpu.Free(tmp)
	return f.Execute(value.Token, tmp, right)
}
