package core

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/cli/q/src/build/errors"
	"git.akyoto.dev/cli/q/src/build/token"
)

// ExecuteRegisterNumber performs an operation on a register and a number.
func (f *Function) ExecuteRegisterNumber(operation token.Token, register cpu.Register, number int) error {
	switch operation.Text() {
	case "+", "+=":
		f.assembler.RegisterNumber(asm.ADD, register, number)

	case "-", "-=":
		f.assembler.RegisterNumber(asm.SUB, register, number)

	case "*", "*=":
		f.assembler.RegisterNumber(asm.MUL, register, number)

	case "/", "/=":
		f.assembler.RegisterNumber(asm.DIV, register, number)

	case "==", "!=", "<", "<=", ">", ">=":
		f.assembler.RegisterNumber(asm.COMPARE, register, number)

	case "=":
		f.assembler.RegisterNumber(asm.MOVE, register, number)

	default:
		return errors.New(&errors.InvalidOperator{Operator: operation.Text()}, f.File, operation.Position)
	}

	return nil
}
